//-----------------------------------------------------------------------------
// VSMSGW.H
//
// Encapsulates sending/receiving of VSMS messages to RTCU Units thru 
// the Logic IO GPRS Gateway
//
//-----------------------------------------------------------------------------

// Different returncodes from the functions:
#define GWRC_OK             0     /* operation successfull */
#define GWRC_ERROR          1     /* unspecified error */
#define GWRC_NOT_CON        2     /* not connected */
#define GWRC_TIMEOUT        3     /* a timeout occured */
#define GWRC_INV_LEN        5     /* invalid length */
#define GWRC_IS_OPEN        7     /* is already open */
#define GWRC_NOT_OPEN       8     /* is not open */
#define GWRC_INV_PARM       9     /* invalid parameters */
#define GWRC_DST_UNREACH    10     /* destination node is unreachable */



//-----------------------------------------------------------------------------
// Definition of callback function for receiving Text SMS messages
// Input :
//   HisNodeID  : The nodeid of the sender (the serialnumber of the RTCU that sent the message)
//   str        : The text string sent by the RTCU.Max size is 160 characters + a null-terminator
//   arg        : A user supplied 32 bit variable that was set when vsmsgwInit() was called
// Output:
//   rc         : This is the return code that should be returned to the RTCU Unit, 0 if OK, else error  
//-----------------------------------------------------------------------------
typedef int(__stdcall *cbfuncText)(int HisNodeID, BSTR str, void *arg);


//-----------------------------------------------------------------------------
// Definition of callback function for receiving PDU SMS messages
// Input :
//   HisNodeID  : The nodeid of the sender (the serialnumber of the RTCU that sent the message)
//   data       : The data sent by the RTCU.Max size is 140 bytes.
//   length     : The length of data sent by the RTCU
//   arg        : A user supplied 32 bit variable that was set when vsmsgwInit() was called
// Output:
//   rc         : This is the return code that should be returned to the RTCU Unit, 0 if OK, else error  
//-----------------------------------------------------------------------------
typedef int(__stdcall *cbfuncPDU)(int HisNodeID, SAFEARRAY **data, int length, void *arg);


//-----------------------------------------------------------------------------
// Definition of callback function for receiving data package messages
// Input :
//   HisNodeID  : The nodeid of the sender (the serialnumber of the RTCU that sent the message)
//   data       : The data sent by the RTCU.Max size is 480 bytes.
//   length     : The length of data sent by the RTCU
//   arg        : A user supplied 32 bit variable that was set when vsmsgwInit() was called
// Output:
//   rc         : This is the return code that should be returned to the RTCU Unit, 0 if OK, else error  
//-----------------------------------------------------------------------------
typedef int(__stdcall *cbWfuncPacksge)(int HisNodeID, SAFEARRAY **data, int length, void *arg);


//-----------------------------------------------------------------------------
// Initialize the connection to the GPRS Gateway
// Input:
//   MyNodeID  : The nodeid for the PC application. If set to 0, it will be assigned by the GPRS Gateway
//   GWIP      : The IP address (or symbolic name) of the GPRS Gateway
//   GWPort    : The pportnumber the GPRS Gateway listens on
//   GWKey     : The key value (an 8 character password) used to access the GPRS Gateway. Must be null terminated,
//   SMSText   : A callbackfunction that will be called whenever an Text SMS is received
//   SMSPDU    : A callbackfunction that will be called whenever an PDU SMS is received
//   arg       : A user supplied 32 bit variable that will be supplied to the callback functions when they get called
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
__declspec(dllimport) int __stdcall vsmsgwInit(int MyNodeID, const char *GWIP, int GWPort, const char *GWKey, cbfuncText SMSText, cbfuncPDU SMSPDU, void *arg);


//-----------------------------------------------------------------------------
// Initialize the connection to the GPRS Gateway
// Input:
//   MyNodeID  : The nodeid for the PC application. If set to 0, it will be assigned by the GPRS Gateway
//   GWIP      : The IP address (or symbolic name) of the GPRS Gateway
//   GWPort    : The pportnumber the GPRS Gateway listens on
//   GWKey     : The key value (an 8 character password) used to access the GPRS Gateway. Must be null terminated,
//   CryptKey  : The encryption key used to access the GPRS Gateway. (16 Bytes)
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
__declspec(dllimport) int __stdcall vsmsgwInitAdv(int MyNodeID, const char *GWIP, int GWPort, const char *GWKey, unsigned char *CryptKey);


//-----------------------------------------------------------------------------
// Set callback function that will be called whenever a Text SMS is received
// Input:
//   SMSText   : A callback function that will be called whenever a Text SMS is received
//   arg       : A user supplied 32 bit variable that will be supplied to the callback functions when they get called
//-----------------------------------------------------------------------------
__declspec(dllimport) void __stdcall vsmsgwSetSMSTextCB(cbWfuncText SMSText, void *arg);


//-----------------------------------------------------------------------------
// Set callback function that will be called whenever a PDU SMS is received
// Input:
//   SMSPDU    : A callback function that will be called whenever a PDU SMS is received
//   arg       : A user supplied 32 bit variable that will be supplied to the callback functions when they get called
//-----------------------------------------------------------------------------
__declspec(dllimport) void __stdcall vsmsgwSetSMSPDUCB(cbWfuncPDU SMSPDU, void *arg);


//-----------------------------------------------------------------------------
// Set callback function that will be called whenever a data package is received
// Input:
//   PACKAGE   : A callback function that will be called whenever a data package is received
//   arg       : A user supplied 32 bit variable that will be supplied to the callback functions when they get called
//-----------------------------------------------------------------------------
__declspec(dllimport) void __stdcall vsmsgwSetDataPackageCB(cbWfuncPacksge PACKAGE, void *arg);


//-----------------------------------------------------------------------------
// Returns true if connected to the GPRS Gateway
//-----------------------------------------------------------------------------
__declspec(dllimport) bool __stdcall vsmsgwIsConnected(void);


//-----------------------------------------------------------------------------
// This function will return this nodes nodeid
// This function is used especially when a dynamic node-id is requested (by 
// setting MyNodeID to 0 in the vsmsgwInit() function
//-----------------------------------------------------------------------------
__declspec(dllimport) int __stdcall vsmsgwGetMyNodeID(unsigned long* MyNodeID);


//-----------------------------------------------------------------------------
// Send a Text SMS message to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc.
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   str       : The string to send to the receiving RTCU unit. Maximum size is 160 characters + NULL terminator.
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
__declspec(dllimport) int __stdcall vsmsgwSendSMS(unsigned int HisNodeID, const char *str,  int *rc);


//-----------------------------------------------------------------------------
// Send a PDU SMS message to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   data      : The data to send to the receiving RTCU unitMaximum size is 140 bytes.
//   length    : The length of data to send
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
__declspec(dllimport) int __stdcall vsmsgwSendPDU(unsigned int HisNodeID, unsigned char *data, int length, int *rc);


//-----------------------------------------------------------------------------
// Send a data package message to the specified NodeID, the returncode from the
// RTCU unit, will be contained in rc
// Input:
//   HisNodeID : The nodenumber (serialnumber) of the receiving RTCU unit
//   data      : The data to send to the receiving RTCU unitMaximum size is 480 bytes.
//   length    : The length of data to send
// Output:
//   rc        : The return code from the receiving RTCU unit, 0 if OK, else error
// Returns 0 if OK, else error (Look at GWRC_xx codes)
//-----------------------------------------------------------------------------
__declspec(dllimport) int __stdcall vsmsgwSendDataPackage(unsigned int HisNodeID, unsigned char *data, int length, int *rc);

